import 'dart:async';
import 'dart:collection';

import 'package:flutter/services.dart';

class NajvaFlutter {
  ///final variables used for method names
  static const String INIT = "init";
  static const String GET_SUBSCRIBED_TOKEN = "get_subscribed_token";
  static const String DISABLE_PUSH_NOTIFICATION = "disablePushNotification";
  static const String DISABLE_LOCATION = "disableLocation";

  ///final variables for dart method names
  static const String NEW_JSON_DATA = "onNewJSONData";
  static const String NEW_USER_SUBSCRIBED = " onUserSubscribed";
  static const String NEW_NOTIFICATION_RECEIVED = "onNotificationReceived";
  static const String NEW_NOTIFICATION_CLICKED = "onNotificationClicked";
  static const String GET_CACHED_JSON_DATA = "getCachedJsonData";

  MethodChannel _channel = const MethodChannel('najvaFlutterplugin');

  bool isNajvaInitialized = false;

  Future<dynamic> handler(MethodCall call) {
    String name = call.method;
    print("NajvaFlutter: ${call.method}");
    switch (name) {
      case NEW_JSON_DATA:
        deliverJSONData(call.arguments);
        break;
      case NEW_USER_SUBSCRIBED:
        deliverUserSubscription(call.arguments);
        break;
      case NEW_NOTIFICATION_CLICKED:
        print("click: ${call.arguments}");
        onNotificationClicked(call.arguments);
        break;
      case NEW_NOTIFICATION_RECEIVED:
        print("receive: ${call.arguments}");
        onNotificationReceived(call.arguments);
        break;
    }
  }

  Future<void> init() async {
    try {
      isNajvaInitialized = await _channel.invokeMethod(INIT);
      _channel.setMethodCallHandler(handler);
    } on PlatformException catch (e) {
      print(e.toString());
    }
  }

  Future<void> disableNotification() async {
    try {
      _channel.invokeMethod(DISABLE_PUSH_NOTIFICATION);
    } on PlatformException catch (e) {
      print(e.toString());
    }
  }

  Future<void> disableLocation() async {
    try {
      _channel.invokeMethod(DISABLE_LOCATION);
    } on PlatformException catch (e) {
      print(e.toString());
    }
  }

  void deliverJSONData(arguments) {
    onNewJSONDataReceived(arguments);
  }

  void deliverUserSubscription(arguments) {
    onUserSubscribed(arguments);
  }

  void onNewJSONDataReceived(String jsonData) {}

  void onUserSubscribed(String token) {}

  Future<String> getSubscribedToken() async {
    String token = await _channel.invokeMethod(GET_SUBSCRIBED_TOKEN);
    return token;
  }

  void getCachedJsonData() {
    _channel.invokeMethod(GET_CACHED_JSON_DATA);
  }

  void onNotificationClicked(LinkedHashMap<dynamic, dynamic> data) {}

  void onNotificationReceived(LinkedHashMap<dynamic, dynamic> data) {}
}
